@@ -1,32 +1,47 @@
 /* collectnews.c -- program to add news articles to relevant header files
- $Id: collectnews.c,v 1.19 1999/10/02 00:43:03 leg Exp $
- 
- # Copyright 1998 Carnegie Mellon University
- # 
- # No warranties, either expressed or implied, are made regarding the
- # operation, use, or results of the software.
- #
- # Permission to use, copy, modify and distribute this software and its
- # documentation is hereby granted for non-commercial purposes only
- # provided that this copyright notice appears in all copies and in
- # supporting documentation.
- #
- # Permission is also granted to Internet Service Providers and others
- # entities to use the software for internal purposes.
- #
- # The distribution, modification or sale of a product which uses or is
- # based on the software, in whole or in part, for commercial purposes or
- # benefits requires specific, additional permission from:
- #
- #  Office of Technology Transfer
- #  Carnegie Mellon University
- #  5000 Forbes Avenue
- #  Pittsburgh, PA  15213-3890
- #  (412) 268-4387, fax: (412) 268-7395
- #  tech-transfer@andrew.cmu.edu
+ $Id: collectnews.c,v 1.24 2000/05/23 20:52:13 robeson Exp $
+ * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
  *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include <config.h>
+
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <syslog.h>
@@ -35,11 +50,14 @@
 #include <unistd.h>
 #endif
 
-#include "config.h"
+#include "imapconf.h"
 #include "exitcodes.h"
 #include "imap_err.h"
 #include "mailbox.h"
 #include "xmalloc.h"
+#include "mboxlist.h"
+#include "append.h"
+#include "convert_code.h"
 
 extern int errno;
 
@@ -89,7 +107,7 @@ int main(int argc, char **argv)
 
 	    if (!strchr(buf, '\t')) {
 		/* No overview junk to throw away */
-		if (p = strrchr(buf, ' ')) {
+		if ((p = strrchr(buf, ' '))!=NULL) {
 		    /* Throw away last (partial) filename */
 		    *p = '\0';
 		}
@@ -146,7 +164,7 @@ int main(int argc, char **argv)
 void collect(char *group, unsigned long feeduid)
 {
     int r;
-    struct mailbox mailbox;
+    struct appendstate as;
     char namebuf[MAX_MAILBOX_PATH];
     struct newsgroup *ng;
     
@@ -169,12 +187,12 @@ void collect(char *group, unsigned long feeduid)
 	}
     }
 
-    r = append_setup(&mailbox, newsprefix ? namebuf : group,
+    r = append_setup(&as, newsprefix ? namebuf : group,
 		     MAILBOX_FORMAT_NETNEWS, 0, 0, 0);
 
     if (r == IMAP_MAILBOX_NONEXISTENT) {
 	r = mboxlist_createmailbox(newsprefix ? namebuf : group,
-				   MAILBOX_FORMAT_NETNEWS, "news",
+				   MBTYPE_NETNEWS, "news",
 				   1, "anonymous", 0);
 
 	/* Ignore bad mailbox names */
@@ -187,7 +205,7 @@ void collect(char *group, unsigned long feeduid)
 	    fatal("cannot create mailbox for new newsgroup", convert_code(r));
 	}
 
-	r = append_setup(&mailbox, newsprefix ? namebuf : group,
+	r = append_setup(&as, newsprefix ? namebuf : group,
 			 MAILBOX_FORMAT_NETNEWS, 0, 0, 0);
     }
 
@@ -202,22 +220,31 @@ void collect(char *group, unsigned long feeduid)
      * Avoid O(n**2) behavior when we're indexing articles
      * that have already expired.
      */
-    if (mailbox.last_uid < ng->last_uid) mailbox.last_uid = ng->last_uid;
+    if (as.m.last_uid < ng->last_uid) as.m.last_uid = ng->last_uid;
 
-    r = append_collectnews(&mailbox, group, feeduid);
+    r = append_collectnews(&as, group, feeduid);
 
     if (r) {
+	append_abort(&as);
 	syslog(LOG_CRIT, "cannot append to %s: %s",
 	       newsprefix ? namebuf : group,
 	       error_message(r));
 	fatal("cannot append to mailbox for newsgroup", convert_code(r));
     }
 
-    ng->last_uid = mailbox.last_uid; 
+    append_commit(&as, NULL, NULL, NULL);
+    ng->last_uid = as.m.last_uid; 
     if (ng->last_uid < feeduid) ng->last_uid = feeduid;
 
-    mailbox_expungenews(&mailbox);
-    mailbox_close(&mailbox);
+    /* now expunge old messages */
+    {
+	struct mailbox mailbox;
+
+	mailbox_open_header(newsprefix ? namebuf : group, NULL,
+			    &mailbox);
+	mailbox_expungenews(&mailbox);
+	mailbox_close(&mailbox);
+    }
 }
 
 #define GROW 1000
