@@ -41,7 +41,7 @@
  *
  */
 /*
- * $Id: index.c,v 1.114 2000/06/07 20:56:04 leg Exp $
+ * $Id: index.c,v 1.120 2000/06/12 01:30:41 leg Exp $
  */
 #include <config.h>
 
@@ -74,31 +74,6 @@
 #include "message.h"
 #include "parseaddr.h"
 
-#ifdef ENABLE_REGEX
-#ifdef HAVE_RX
-#include <rxposix.h>
-#else
-#include <regex.h>
-#endif
-
-/* Subject extraction grammar (SORT/THREAD).
- *
- * This grammar deviates from the one shown in the draft, but coupled
- * with the logic in extract_subject(), it is functionally equivalent.
- * Also note that we optionally allow the leading and trailing double-quotes
- * that are present in the cached subject header.
- */
-#define TRAILER		"(" "\\(fwd\\)" "|" WSP ")*" "\"?$"
-#define LEADER		"^\"?" "(" REFWD "|" BLOB "|" WSP ")*"
-#define REFWD		"(re|fwd?)" WSP "(" BLOB ")?" WSP ":"
-#define BLOB		"\\[" BLOBCHAR "*\\]"
-#define BLOBCHAR	"[^]]"
-#define WSP		"[\t\r\n ]*"
-
-/* Compiled regexes for subject extraction */
-static regex_t pleader, ptrailer, pblob;
-#endif /* ENABLE_REGEX */
-
 extern int errno;
 
 extern void printastring (const char *s);
@@ -159,10 +134,11 @@ struct mapfile {
     unsigned long size;
 };
 
-struct msgdata {
+typedef struct msgdata {
     unsigned msgno;		/* message number */
-    unsigned msgid;		/* hash of message ID */
-    unsigned *ref;		/* hash of references */
+    char *msgid;		/* message ID */
+    char **ref;			/* array of references */
+    int nref;			/* number of references */
     time_t arrival;		/* internal arrival date & time of message */
     time_t date;		/* sent date & time of message
 				   from Date: header (adjusted by time zone) */
@@ -173,18 +149,18 @@ struct msgdata {
     unsigned xsubj_hash;	/* hash of extracted subject text */
     unsigned size;		/* size of message */
     struct msgdata *next;
-};
+} MsgData;
 
-struct thread {
-    struct msgdata *msgdata;	/* message data */
+typedef struct thread {
+    MsgData *msgdata;		/* message data */
     struct thread *parent;	/* parent message */
     struct thread *child;	/* first child message */
     struct thread *next;	/* next sibling message */
-};
+} Thread;
 
 struct thread_algorithm {
     char *alg_name;
-    struct thread *(*threader)(unsigned *msgno_list, int nmsg);
+    void (*threader)(unsigned *msgno_list, int nmsg, int usinguid);
 };
 
 /* Forward declarations */
@@ -240,25 +216,23 @@ static index_sequenceproc_t index_copysetup;
 static int _index_search P((unsigned **msgno_list, struct mailbox *mailbox,
 			    struct searchargs *searchargs));
 
-static char *index_extract_subject(const char *subj);
-static struct msgdata *index_msgdata_load P((unsigned *msgno_list, int n,
-					     signed char *sortcrit));
+static char *index_extract_subject P((const char *subj));
+static MsgData *index_msgdata_load P((unsigned *msgno_list, int n,
+				      signed char *sortcrit));
 
-static void *index_sort_getnext P((struct msgdata *node));
-static void index_sort_setnext P((struct msgdata *node, struct msgdata *next));
-static int index_sort_compare P((struct msgdata *md1, struct msgdata *md2,
+static void *index_sort_getnext P((MsgData *node));
+static void index_sort_setnext P((MsgData *node, MsgData *next));
+static int index_sort_compare P((MsgData *md1, MsgData *md2,
 				 signed char *call_data));
-static void index_sort_free P((struct msgdata *list));
+static void index_msgdata_free P((MsgData *md));
 
-static void *index_thread_getnext P((struct thread *thread));
-static void index_thread_setnext P((struct thread *thread,
-				    struct thread *next));
-static int index_thread_compare P((struct thread *t1, struct thread *t2,
+static void *index_thread_getnext P((Thread *thread));
+static void index_thread_setnext P((Thread *thread, Thread *next));
+static int index_thread_compare P((Thread *t1, Thread *t2,
 				   signed char *call_data));
-static struct thread *index_thread_orderedsubj P((unsigned *msgno_list,
-						  int nmsg));
-static void index_thread_print P((struct thread *threads, int usinguid));
-static void index_thread_free P((struct thread *threads));
+static void index_thread_orderedsubj P((unsigned *msgno_list, int nmsg,
+					int usinguid));
+static void index_thread_print P((Thread *threads, int usinguid));
 
 static struct thread_algorithm thread_algs[] = {
     { "ORDEREDSUBJECT", index_thread_orderedsubj },
@@ -307,11 +281,7 @@ int examine_mode;
 /*
  * Check for and report updates
  */
-void
-index_check(mailbox, usinguid, checkseen)
-struct mailbox *mailbox;
-int usinguid;
-int checkseen;
+void index_check(struct mailbox *mailbox, int usinguid, int checkseen)
 {
     struct stat sbuf;
     int newexists, oldexists, oldmsgno, msgno, nexpunge, i, r;
@@ -573,7 +543,7 @@ int oldexists;
 	if (oldseen != newseen) {
 	    if (seenflag[msgno] != newseen) {
 		seenflag[msgno] = newseen;
-		if (!quiet && msgno <= oldexists) {
+		if (!quiet && msgno <= oldexists && oldexists != -1) {
 		    for (i = 0; i < MAX_USER_FLAGS/32; i++) {
 			user_flags[i] = USER_FLAGS(msgno, i);
 		    }
@@ -1027,7 +997,7 @@ index_sort(struct mailbox *mailbox,
 	   int usinguid)
 {
     unsigned *msgno_list;
-    struct msgdata *msgdata = NULL, *cur;
+    MsgData *msgdata = NULL, *freeme = NULL, *cur;
     int n;
 
     /* Search for messages based on the given criteria */
@@ -1035,7 +1005,7 @@ index_sort(struct mailbox *mailbox,
 
     if (n) {
 	/* Create/load the msgdata array */
-	msgdata = index_msgdata_load(msgno_list, n, sortcrit);
+	freeme = msgdata = index_msgdata_load(msgno_list, n, sortcrit);
 	free(msgno_list);
     }
 
@@ -1048,14 +1018,21 @@ index_sort(struct mailbox *mailbox,
 			(void (*)(void*,void*)) index_sort_setnext,
 			(int (*)(void*,void*,void*)) index_sort_compare,
 			sortcrit);
+
 	/* Output the sorted messages */ 
 	cur = msgdata;
 	while (cur) {
 	    prot_printf(imapd_out, " %u",
 			usinguid ? UID(cur->msgno) : cur->msgno);
+
+	    /* free contents of the current node */
+	    index_msgdata_free(cur);
+
 	    cur = cur->next;
 	}
-	index_sort_free(msgdata);
+
+	/* free the msgdata array */
+	free(freeme);
     }
 
     prot_printf(imapd_out, "\r\n");
@@ -1067,7 +1044,6 @@ index_sort(struct mailbox *mailbox,
 void index_thread(struct mailbox *mailbox, int algorithm,
 		  struct searchargs *searchargs, int usinguid)
 {
-    struct thread *threads;
     unsigned *msgno_list;
     int nmsg;
 
@@ -1075,19 +1051,10 @@ void index_thread(struct mailbox *mailbox, int algorithm,
     nmsg = _index_search(&msgno_list, mailbox, searchargs);
 
     if (nmsg) {
-	/* Thread messages using given algorithm. */
-	threads = (*thread_algs[algorithm].threader)(msgno_list, nmsg);
-
-	/* Output the threaded messages */ 
-	prot_printf(imapd_out, "* THREAD ");
-
-	index_thread_print(threads, usinguid);
-
-	prot_printf(imapd_out, "\r\n");
+	/* Thread messages using given algorithm */
+	(*thread_algs[algorithm].threader)(msgno_list, nmsg, usinguid);
 
 	free(msgno_list);
-
-	index_thread_free(threads);
     }
 }
 
@@ -2108,9 +2075,7 @@ char *trail;
 /*
  * Send a * FLAGS response.
  */
-static void
-index_listflags(mailbox)
-struct mailbox *mailbox;
+static void index_listflags(struct mailbox *mailbox)
 {
     int i;
     int cancreate = 0;
@@ -2989,13 +2954,8 @@ void *rock;
  *
  * public domain code by Jerry Coffin, with improvements by HenkJan Wolthuis.
  *
- * Questions:
- *
- * Does this function provide a unique hash for all strings?  If not, either
- * find one that does, or just use strcmp().
- *
- * Can we find a function that provides uniqueness and maintains alphabetical
- * (ascii) ordering between strings ( hash(abc) < hash(abd) < hash(abe) )?
+ * Can we find a function that maintains alphabetical (ascii) ordering
+ * between strings ( hash(abc) < hash(abd) < hash(abe) )?
  */
 
 static unsigned hash(char *string)
@@ -3004,7 +2964,7 @@ static unsigned hash(char *string)
     int i;
 
     while (*string) {
-	i = *(int *) string;
+	i = (int) *string;
 	ret_val ^= i;
 	ret_val <<= 1;
 	string ++;
@@ -3019,20 +2979,16 @@ static unsigned hash(char *string)
  * by the specified sort criteria.
  */
 
-#define NEWMSGDATA \
-    (struct msgdata *) memset(xmalloc(sizeof(struct msgdata)), 0, \
-			      sizeof(struct msgdata))
-
 #define GETADDR(header)							\
     parseaddr_list((header)+4, &addr);					\
     cur->header = xstrdup(addr && addr->mailbox ? addr->mailbox : "");	\
     parseaddr_free(addr);						\
     addr=NULL
 
-static struct msgdata *index_msgdata_load(unsigned *msgno_list, int n,
-					  signed char *sortcrit)
+static MsgData *index_msgdata_load(unsigned *msgno_list, int n,
+				   signed char *sortcrit)
 {
-    struct msgdata *md, *cur;
+    MsgData *md, *cur;
     const char *cacheitem, *env, *from, *to, *cc, *subj;
     struct address *addr;
     int i, j;
@@ -3040,12 +2996,16 @@ static struct msgdata *index_msgdata_load(unsigned *msgno_list, int n,
     if (!n)
 	return NULL;
 
-    cur = md = NEWMSGDATA;
+    /* create an array of MsgData to use as nodes of linked list */
+    md = (MsgData *) memset(xmalloc(n * sizeof(MsgData)), 0,
+			    n * sizeof(MsgData));
 
-    i = 0;
-    do {
+    for (i = 0, cur = md; i < n; i++, cur = cur->next) {
 	/* set msgno */
-	cur->msgno = msgno_list[i++];
+	cur->msgno = msgno_list[i];
+
+	/* set pointer to next node */
+	cur->next = (i+1 < n ? cur+1 : NULL);
 
 	/* fetch cached info */
 	env = cache_base + CACHE_OFFSET(cur->msgno);
@@ -3070,8 +3030,8 @@ static struct msgdata *index_msgdata_load(unsigned *msgno_list, int n,
 		break;
 	    case SORT_DATE:
 		/* skip open paren and double-quote */
-		cur->date = message_parse_date((char*) env+6,
-					      PARSE_TIME | PARSE_ZONE);
+		cur->date = message_parse_date((char *) env+6,
+					       PARSE_TIME | PARSE_ZONE);
 		break;
 	    case SORT_FROM:
 		GETADDR(from);
@@ -3088,7 +3048,7 @@ static struct msgdata *index_msgdata_load(unsigned *msgno_list, int n,
 		break;
 	    }
 	}
-    } while (i < n && (cur = cur->next = NEWMSGDATA));
+    }
 
     return md;
 }
@@ -3098,8 +3058,14 @@ char *index_extract_subject(const char *subj)
 {
     char *s, *base, *ret, *x;
 
-    /* make a working copy of subj */
-    s = xstrdup(subj);
+    if (!strcmp(subj, "NIL"))				/* "NIL"? */
+	return xstrdup("");				/* yes, return empty */
+
+    /* make a working copy of subj -- remove double-quotes, if they exist */
+    if (*subj == '"')
+	s = xstrndup(subj+1, strlen(subj) - 2);
+    else
+	s = xstrdup(subj);
 
     /* trim trailer
      *
@@ -3107,7 +3073,7 @@ char *index_extract_subject(const char *subj)
      * resetting the end of the string as we go.
      */
     for (x = s + strlen(s) - 1; x >= s;) {
-	if (isspace(*x)) {				/* whitespace? */
+	if (isspace((int) *x)) {                        /* whitespace? */
 	    *x = '\0';					/* yes, trim it */
 	    x--;					/* skip past it */
 	}
@@ -3126,8 +3092,7 @@ char *index_extract_subject(const char *subj)
      * skipping over stuff we don't care about.
      */
     for (base = s; base;) {
-	if (isspace(*base))				/* whitespace? */
-	    base++;					/* yes, skip past it */
+	if (isspace((int) *base)) base++;		/* whitespace? */
 
 	/* possible refwd */
 	else if ((!strncasecmp(base, "re", 2) &&	/* "re"? */
@@ -3137,18 +3102,17 @@ char *index_extract_subject(const char *subj)
 		 (!strncasecmp(base, "fw", 2) &&	/* "fw"? */
 		  (x = base + 2))) {			/* yes, skip past it */
 	    
-	    while (isspace(*x))				/* whitespace? */
-		x++;					/* yes, skip past it */
+	    while (isspace((int) *x)) x++;		/* skip whitespace */
 
 	    if (*x == '[') {				/* start of blob? */
-		if (x = strchr(x, ']'))			/* yes, end of blob? */
+		x = strchr(x, ']');
+		if (x)			                /* yes, end of blob? */
 		    x++;				/* yes, skip past it */
 		else
 		    break;				/* no, we're done */
 	    }
 
-	    while (isspace(*x))				/* whitespace? */
-		x++;					/* yes, skip past it */
+	    while (isspace((int) *x)) x++;              /* skip whitespace */
 
 	    if (*x == ':')				/* ending colon? */
 		base = x + 1;				/* yes, skip past it */
@@ -3192,7 +3156,7 @@ char *index_extract_subject(const char *subj)
 /*
  * Getnext function for sorting message lists.
  */
-static void *index_sort_getnext(struct msgdata *node)
+static void *index_sort_getnext(MsgData *node)
 {
     return node->next;
 }
@@ -3200,7 +3164,7 @@ static void *index_sort_getnext(struct msgdata *node)
 /*
  * Setnext function for sorting message lists.
  */
-static void index_sort_setnext(struct msgdata *node, struct msgdata *next)
+static void index_sort_setnext(MsgData *node, MsgData *next)
 {
     node->next = next;
 }
@@ -3213,7 +3177,7 @@ static int numcmp(int x, int y)
     return ((x < y) ? -1 : (x > y) ? 1 : 0);
 }
 
-static int index_sort_compare(struct msgdata *md1, struct msgdata *md2,
+static int index_sort_compare(MsgData *md1, MsgData *md2,
 			      signed char *sortcrit)
 {
     int reverse, ret = 0, i = 0;
@@ -3263,35 +3227,19 @@ static int index_sort_compare(struct msgdata *md1, struct msgdata *md2,
 /*
  * Free a msgdata node.
  */
-static void index_msgdata_free(struct msgdata *md)
+static void index_msgdata_free(MsgData *md)
 {
 #define FREE(x)	if (x) free(x)
     FREE(md->cc);
     FREE(md->from);
     FREE(md->to);
     FREE(md->xsubj);
-    free(md);
-}
-
-/*
- * Free a list of sorted messages.
- */
-static void index_sort_free(struct msgdata *list)
-{
-
-    struct msgdata *tmp;
-
-    while (list) {
-	tmp = list;
-	list = list->next;
-	index_msgdata_free(tmp);
-    }
 }
 
 /*
  * Getnext function for sorting thread lists.
  */
-static void *index_thread_getnext(struct thread *thread)
+static void *index_thread_getnext(Thread *thread)
 {
     return thread->next;
 }
@@ -3299,7 +3247,7 @@ static void *index_thread_getnext(struct thread *thread)
 /*
  * Setnext function for sorting thread lists.
  */
-static void index_thread_setnext(struct thread *thread, struct thread *next)
+static void index_thread_setnext(Thread *thread, Thread *next)
 {
     thread->next = next;
 }
@@ -3307,7 +3255,7 @@ static void index_thread_setnext(struct thread *thread, struct thread *next)
 /*
  * Comparison function for sorting threads.
  */
-static int index_thread_compare(struct thread *t1, struct thread *t2,
+static int index_thread_compare(Thread *t1, Thread *t2,
 				signed char *call_data)
 {
   return index_sort_compare(t1->msgdata, t2->msgdata, call_data);
@@ -3316,20 +3264,17 @@ static int index_thread_compare(struct thread *t1, struct thread *t2,
 /*
  * Thread a list of messages using the ORDEREDSUBJECT algorithm.
  */
-#define NEWTHREAD \
-    (struct thread *) memset(xmalloc(sizeof(struct thread)), 0, \
-			     sizeof(struct thread))
-
-static struct thread *index_thread_orderedsubj(unsigned *msgno_list, int nmsg)
+static void index_thread_orderedsubj(unsigned *msgno_list, int nmsg,
+				     int usinguid)
 {
-    struct msgdata *msgdata;
+    MsgData *msgdata, *freeme;
     signed char sortcrit[] = { SORT_SUBJECT, SORT_DATE, SORT_SEQUENCE };
     unsigned psubj_hash;
     char *psubj;
-    struct thread *threads, *tmp, *cur, *parent;
+    Thread *head, *newnode, *cur, *parent, *threads;
 
     /* Create/load the msgdata array */
-    msgdata = index_msgdata_load(msgno_list, nmsg, sortcrit);
+    freeme = msgdata = index_msgdata_load(msgno_list, nmsg, sortcrit);
 
     /* Sort messages by subject and date */
     msgdata = lsort(msgdata,
@@ -3338,53 +3283,70 @@ static struct thread *index_thread_orderedsubj(unsigned *msgno_list, int nmsg)
 		    (int (*)(void*,void*,void*)) index_sort_compare,
 		    sortcrit);
 
-    /* build threads under a dummy head */
-    tmp = cur = NEWTHREAD;
-    parent = NULL;
-    psubj_hash = msgdata->xsubj_hash + 1; /* guarantee psubj != first subj */
-    psubj = NULL;
-    while (msgdata) {
-	/* if current subj == prev subj, then add message to current thread */
-	if (msgdata->xsubj_hash == psubj_hash &&
-	    !strcmp(msgdata->xsubj, psubj)) { /* if 2 subjs have same key */
-	    parent->child = NEWTHREAD;
-	    parent = parent->child; /* this'll be the parent 
-				       next time around */
-	    parent->msgdata = msgdata;
-	}
+    /* create an array of Thread to use as nodes of thread tree
+     *
+     * we will be building threads under a dummy head,
+     * so we need (nmsg + 1) nodes
+     */
+    head = (Thread *) xmalloc((nmsg + 1) * sizeof(Thread));
+    memset(head, 0, (nmsg + 1) * sizeof(Thread));
 
+    cur = head;		/* set current thread to the dummy head */
+    newnode = head + 1;	/* set next newnode to the second one in the array */
+
+    /* guarantee psubj != first subj so that we start a new thread */
+    psubj_hash = msgdata->xsubj_hash + 1;
+    psubj = NULL;
+    parent = NULL;
+
+    while (msgdata) {
+	/* if current subj = prev subj (subjs have same hash, and
+	   the strings are equal), then add message to current thread */
+	if (msgdata->xsubj_hash == psubj_hash &&
+	    !strcmp(msgdata->xsubj, psubj)) {
+	    parent->child = newnode;	/* create a new child */
+	    parent->child->msgdata = msgdata;
+	    parent = parent->child;	/* this'll be the parent 
+					   next time around */
+	}
 	/* otherwise, create a new thread */
 	else {
-	    cur->next = NEWTHREAD; /* create a new entry */
-	    parent = cur = cur->next; /* now work with the new entry */
-	    cur->msgdata = msgdata;
+	    cur->next = newnode;	/* create and start a new thread */
+	    cur->next->msgdata = msgdata;
+	    parent = cur = cur->next;	/* now work with the new thread */
 	}
 
 	psubj_hash = msgdata->xsubj_hash;
 	psubj = msgdata->xsubj;
 	msgdata = msgdata->next;
+	newnode++;
     }
 
     /* Sort threads by date */
-    threads = lsort(tmp->next,
+    threads = lsort(head->next,
 		    (void * (*)(void*)) index_thread_getnext,
 		    (void (*)(void*,void*)) index_thread_setnext,
 		    (int (*)(void*,void*,void*)) index_thread_compare,
 		    sortcrit+1);
 
-    /* free the dummy head */
-    free(tmp);
+    /* Output the threaded messages */ 
+    index_thread_print(threads, usinguid);
 
-    return threads;
+    /* free the thread array */
+    free(head);
 
+    /* free the msgdata array */
+    free(freeme);
 }
 
 /*
- * Print a list of threads.
+ * Guts of thread printing.  Recurses over children when necessary.
+ *
+ * Frees contents of msgdata as a side effect.
  */
-static void index_thread_print(struct thread *thread, int usinguid)
+static void _index_thread_print(Thread *thread, int usinguid)
 {
-    struct thread *child;
+    Thread *child;
 
     /* for each thread... */
     while (thread) {
@@ -3394,6 +3356,9 @@ static void index_thread_print(struct thread *thread, int usinguid)
 		    usinguid ? UID(thread->msgdata->msgno) :
 		    thread->msgdata->msgno);
 
+	/* free contents of the current node */
+	index_msgdata_free(thread->msgdata);
+
 	/* for each child, grandchild, etc... */
 	child = thread->child;
 	while (child) {
@@ -3409,6 +3374,10 @@ static void index_thread_print(struct thread *thread, int usinguid)
 		prot_printf(imapd_out, "%u",
 			    usinguid ? UID(child->msgdata->msgno) :
 			    child->msgdata->msgno);
+
+		/* free contents of the current node */
+		index_msgdata_free(child->msgdata);
+
 		child = child->child;
 	    }
 	}
@@ -3419,37 +3388,18 @@ static void index_thread_print(struct thread *thread, int usinguid)
 }
 
 /*
- * Free a list of threads.
+ * Print a list of threads.
+ *
+ * This is a wrapper around _index_thread_print() which simply prints the
+ * start and end of the untagged thread response.
  */
-static void index_thread_free(struct thread *threads)
+static void index_thread_print(Thread *thread, int usinguid)
 {
-    struct thread *child, *tmp;
+    prot_printf(imapd_out, "* THREAD ");
 
-    /* for each thread... */
-    while (threads) {
+    _index_thread_print(thread, usinguid);
 
-	/* for each child, grandchild, etc... */
-	child = threads->child;
-	while (child) {
-
-	    /* if the child has siblings, free new branch and break */
-	    if (child->next) {
-		index_thread_free(child);
-		break;
-	    }
-	    /* otherwise free the only child */
-	    else {
-		tmp = child;
-		child = child->child;
-		index_msgdata_free(tmp->msgdata);
-		free(tmp);
-	    }
-	}
-	tmp = threads;
-	threads = threads->next;
-	index_msgdata_free(tmp->msgdata);
-	free(tmp);
-    }
+    prot_printf(imapd_out, "\r\n");
 }
 
 /*
@@ -3478,38 +3428,3 @@ int find_thread_algorithm(char *arg)
     }
     return -1;
 }
-
-/*
- * Compile regexes for subject extraction (SORT/THREAD)
- */
-int sort_thread_enabled()
-{
-    static int isenabled = 0;
-
-#ifdef ENABLE_REGEX
-    if (!isenabled) {
-	int ret;
-
-	ret = regcomp(&ptrailer, TRAILER, REG_ICASE | REG_EXTENDED);
-	ret |= regcomp(&pleader, LEADER, REG_ICASE | REG_EXTENDED);
-	if (ret |= regcomp(&pblob, BLOB, REG_EXTENDED))
-	    fatal("SORT/THREAD regexes failed to compile", EC_SOFTWARE);
-	else
-	    isenabled++;
-    }
-#endif
-
-    return isenabled;
-}
-
-/*
- * Cleanup regexes for subject extraction (SORT/THREAD)
- */
-void sort_thread_cleanup()
-{
-#ifdef ENABLE_REGEX
-    regfree(&ptrailer);
-    regfree(&pleader);
-    regfree(&pblob);
-#endif
-}
