@@ -40,7 +40,7 @@
  */
 
 /*
- * $Id: pop3d.c,v 1.74 2000/06/06 00:51:52 leg Exp $
+ * $Id: pop3d.c,v 1.75 2000/06/09 02:44:53 leg Exp $
  */
 #include <config.h>
 
@@ -139,6 +139,8 @@ static void cmdloop(void);
 void kpop(void);
 static int parsenum(char **ptr);
 void usage(void);
+void shut_down(int code) __attribute__ ((noreturn));
+
 
 extern void setproctitle_init(int argc, char **argv, char **envp);
 extern int proc_register(char *progname, char *clienthost, 
@@ -162,6 +164,9 @@ int service_init(int argc, char **argv, char **envp)
     if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
     setproctitle_init(argc, argv, envp);
 
+    /* set signal handlers */
+    signals_set_shutdown(&shut_down);
+    signals_add_handlers();
     signal(SIGPIPE, SIG_IGN);
 
     /* set the SASL allocation functions */
@@ -196,6 +201,8 @@ int service_main(int argc, char **argv, char **envp)
     int timeout;
     sasl_security_properties_t *secprops=NULL;
 
+    signals_poll();
+
     popd_in = prot_new(0, 0);
     popd_out = prot_new(1, 1);
 
@@ -270,6 +277,13 @@ int service_main(int argc, char **argv, char **envp)
     return 0;
 }
 
+/* called if 'service_init()' was called but not 'service_main()' */
+void service_abort(int error)
+{
+    mboxlist_close();
+    mboxlist_done();
+}
+
 void usage(void)
 {
     prot_printf(popd_out, "-ERR usage: pop3d [-k] [-s]\r\n");
@@ -280,8 +294,6 @@ void usage(void)
 /*
  * Cleanly shut down and exit
  */
-void shut_down(int code) __attribute__ ((noreturn));
-
 void shut_down(int code)
 {
     proc_cleanup();
@@ -372,6 +384,8 @@ static void cmdloop(void)
     unsigned msg = 0;
 
     for (;;) {
+	signals_poll();
+
 	if (!prot_fgets(inputbuf, sizeof(inputbuf), popd_in)) {
 	    shut_down(0);
 	}
