@@ -1,32 +1,52 @@
 /* reconstruct.c -- program to reconstruct a mailbox 
  *
- * Copyright 1998 Carnegie Mellon University
- * 
- * No warranties, either expressed or implied, are made regarding the
- * operation, use, or results of the software.
+ * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
  *
- * Permission to use, copy, modify and distribute this software and its
- * documentation is hereby granted for non-commercial purposes only
- * provided that this copyright notice appears in all copies and in
- * supporting documentation.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * Permission is also granted to Internet Service Providers and others
- * entities to use the software for internal purposes.
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
  *
- * The distribution, modification or sale of a product which uses or is
- * based on the software, in whole or in part, for commercial purposes or
- * benefits requires specific, additional permission from:
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
  *
- *  Office of Technology Transfer
- *  Carnegie Mellon University
- *  5000 Forbes Avenue
- *  Pittsburgh, PA  15213-3890
- *  (412) 268-4387, fax: (412) 268-7395
- *  tech-transfer@andrew.cmu.edu
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: reconstruct.c,v 1.42 1999/08/20 19:05:30 leg Exp $ */
+/* $Id: reconstruct.c,v 1.53 2000/05/23 20:52:31 robeson Exp $ */
 
+#include <config.h>
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <fcntl.h>
@@ -36,6 +56,7 @@
 #include <netinet/in.h>
 #include <sys/stat.h>
 #include <com_err.h>
+#include <stdlib.h>
 
 #if HAVE_DIRENT_H
 # include <dirent.h>
@@ -58,42 +79,63 @@
 #include "assert.h"
 #include "bsearch.h"
 #include "imparse.h"
-#include "config.h"
+#include "imapconf.h"
 #include "exitcodes.h"
 #include "imap_err.h"
 #include "mailbox.h"
 #include "message.h"
 #include "xmalloc.h"
+#include "imapconf.h"
+#include "mboxname.h"
+#include "mboxlist.h"
+#include "seen.h"
+#include "retry.h"
+#include "convert_code.h"
+#include "util.h"
+#include "acapmbox.h"
 
 extern int errno;
 extern int optind;
 extern char *optarg;
 
-extern char *mailbox_findquota P((const char *name));
+struct discovered {
+    char *name;
+    struct discovered *next;
+};       
+
+/* forward declarations */
+void do_mboxlist(void);
+int do_reconstruct(char *name, int matchlen, int maycreate, void *rock);
+int reconstruct(char *name, struct discovered *l);
+void usage(void);
+void shut_down(int code);
+
+extern char *mailbox_findquota(const char *name);
 extern acl_canonproc_t mboxlist_ensureOwnerRights;
 
+static acapmbox_handle_t *acaphandle = NULL;
+
 int code = 0;
 
-int do_reconstruct();
-
-main(argc, argv)
-int argc;
-char **argv;
+int main(int argc, char **argv)
 {
     int opt, i;
     int rflag = 0;
     int mflag = 0;
+    int fflag = 0;
     char buf[MAX_MAILBOX_PATH];
+    struct discovered head;
 
+    memset(&head, 0, sizeof(head));
     config_init("reconstruct");
 
     if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
 
     /* Ensure we're up-to-date on the index file format */
-    assert(INDEX_HEADER_SIZE == (OFFSET_UIDVALIDITY+4));
+    assert(INDEX_HEADER_SIZE == (OFFSET_FLAGGED+4));
     assert(INDEX_RECORD_SIZE == (OFFSET_USER_FLAGS+MAX_USER_FLAGS/8));
 
-    while ((opt = getopt(argc, argv, "rm")) != EOF) {
+    while ((opt = getopt(argc, argv, "rmf")) != EOF) {
 	switch (opt) {
 	case 'r':
 	    rflag = 1;
@@ -103,42 +145,74 @@ char **argv;
 	    mflag = 1;
 	    break;
 
+	case 'f':
+	    fflag = 1;
+	    break;
+
 	default:
 	    usage();
 	}
     }
 
     if (mflag) {
-	if (rflag || optind != argc) usage();
+	if (rflag || fflag || optind != argc) usage();
 	do_mboxlist();
     }
 
+    mboxlist_init(0);
+    mboxlist_open(NULL);
+
+    signals_set_shutdown(&shut_down);
+    signals_add_handlers();
+
     mailbox_reconstructmode();
 
     if (optind == argc) {
-	if (!rflag) usage();
+	if (rflag) {
+	    fprintf(stderr, "please specify a mailbox to recurse from\n");
+	    exit(EC_USAGE);
+	}
+	assert(!rflag);
 	strcpy(buf, "*");
 	mboxlist_findall(buf, 1, 0, 0, do_reconstruct, NULL);
     }
 
     for (i = optind; i < argc; i++) {
+	do_reconstruct(argv[i], 0, 0, fflag ? &head : NULL);
 	if (rflag) {
 	    strcpy(buf, argv[i]);
 	    strcat(buf, ".*");
-	    mboxlist_findall(argv[i], 1, 0, 0, do_reconstruct, NULL);
-	    mboxlist_findall(buf, 1, 0, 0, do_reconstruct, NULL);
-	}
-	else {
-	    do_reconstruct(argv[i], 0, 0);
+	    mboxlist_findall(buf, 1, 0, 0, do_reconstruct, 
+			     fflag ? &head : NULL);
 	}
     }
 
+    /* examine our list to see if we discovered anything */
+    while (head.next) {
+	struct discovered *p;
+	int r;
+
+	p = head.next;
+	head.next = p->next;
+
+	/* create p and reconstruct it */
+	r = mboxlist_createmailbox(p->name, 0, NULL, 1, "cyrus", NULL);
+	if (!r) do_reconstruct(p->name, 0, 0, &head);
+	/* may have added more things into our list */
+
+	free(p->name);
+	free(p);
+    }
+
+    mboxlist_close();
+    mboxlist_done();
+
     exit(code);
 }
 
-usage()
+void usage(void)
 {
-    fprintf(stderr, "usage: reconstruct [-r] mailbox...\n");
+    fprintf(stderr, "usage: reconstruct [-r] [-f] mailbox...\n");
     fprintf(stderr, "       reconstruct -m\n");
     exit(EC_USAGE);
 }    
@@ -151,19 +225,17 @@ char *a, *b;
 
 #define UIDGROW 300
 
+
 /*
  * mboxlist_findall() callback function to reconstruct a mailbox
  */
 int
-do_reconstruct(name, matchlen, maycreate, rock)
-char *name;
-int matchlen;
-int maycreate;
-void* rock;
+do_reconstruct(char *name, int matchlen, int maycreate, void *rock)
 {
     int r;
 
-    r = reconstruct(name);
+    signals_poll();
+    r = reconstruct(name, rock);
     if (r) {
 	com_err(name, r, (r == IMAP_IOERROR) ? error_message(errno) : NULL);
 	code = convert_code(r);
@@ -178,9 +250,7 @@ void* rock;
 /*
  * Reconstruct the single mailbox named 'name'
  */
-int 
-reconstruct(name)
-char *name;
+int reconstruct(char *name, struct discovered *found)
 {
     int r;
     struct mailbox mailbox;
@@ -199,7 +269,11 @@ char *name;
     int uid_num, uid_alloc;
     DIR *dirp;
     struct dirent *dirent;
-    int msg, old_msg = 0, new_exists = 0;
+    int msg, old_msg = 0;
+    int new_exists = 0, 
+	new_answered = 0,
+	new_flagged = 0,
+	new_deleted = 0;
     unsigned long new_quota = 0;
     struct index_record message_index, old_index;
     static struct index_record zero_index;
@@ -311,9 +385,9 @@ char *name;
 	}
 
 	group = mailbox.name;
-	if (newsprefixlen = strlen(config_getstring("newsprefix", ""))) {
+	if ((newsprefixlen = strlen(config_getstring("newsprefix", "")))!=0) {
 	    group += newsprefixlen;
-	    if (*group == '.') group++;
+	    if ((*group) == '.') group++;
 	}
 	strcpy(end_newspath, group);
 
@@ -344,8 +418,8 @@ char *name;
 	    return IMAP_IOERROR;
 	}
     } else {
-	while (dirent = readdir(dirp)) {
-	    if (!isdigit(dirent->d_name[0]) || dirent->d_name[0] ==
+	while ((dirent = readdir(dirp))!=NULL) {
+	    if (!isdigit((int) (dirent->d_name[0])) || dirent->d_name[0] ==
 		'0')
 		continue;
 	    if (uid_num == uid_alloc) {
@@ -355,7 +429,7 @@ char *name;
 	    }
 	    uid[uid_num] = 0;
 	    p = dirent->d_name;
-	    while (isdigit(*p)) {
+	    while (isdigit((int) *p)) {
 		uid[uid_num] = uid[uid_num] * 10 + *p++ - '0';
 	    }
 	    if (format != MAILBOX_FORMAT_NETNEWS) {
@@ -381,7 +455,7 @@ char *name;
 	message_index.uid = uid[msg];
 	
 	if (format == MAILBOX_FORMAT_NETNEWS) {
-	    sprintf(end_newspath, "%u", uid[msg]);
+	    sprintf(end_newspath, "%u", (unsigned int) uid[msg]);
 	    msgfile = fopen(newspath, "r");
 	}
 	else {
@@ -430,7 +504,7 @@ char *name;
 	}
 	message_index.last_updated = time(0);
 	
-	if (r = message_parse_file(msgfile, &mailbox, &message_index)) {
+	if ((r = message_parse_file(msgfile, &mailbox, &message_index))!=0) {
 	    fclose(msgfile);
 	    fclose(newindex);
 	    mailbox_close(&mailbox);
@@ -461,6 +535,9 @@ char *name;
 	}
 
 	new_exists++;
+	if (message_index.system_flags & FLAG_ANSWERED) new_answered++;
+	if (message_index.system_flags & FLAG_FLAGGED) new_flagged++;
+	if (message_index.system_flags & FLAG_DELETED) new_deleted++;
 	new_quota += message_index.size;
     }
     
@@ -488,6 +565,9 @@ char *name;
     *((bit32 *)(buf+OFFSET_QUOTA_MAILBOX_USED)) = htonl(new_quota);
     *((bit32 *)(buf+OFFSET_POP3_LAST_LOGIN)) = htonl(mailbox.pop3_last_login);
     *((bit32 *)(buf+OFFSET_UIDVALIDITY)) = htonl(mailbox.uidvalidity);
+    *((bit32 *)(buf+OFFSET_DELETED)) = htonl(new_deleted);
+    *((bit32 *)(buf+OFFSET_ANSWERED)) = htonl(new_answered);
+    *((bit32 *)(buf+OFFSET_FLAGGED)) = htonl(new_flagged);
 
     n = fwrite(buf, 1, INDEX_HEADER_SIZE, newindex);
     fflush(newindex);
@@ -514,394 +594,77 @@ char *name;
 	return IMAP_IOERROR;
     }
     
-    toimsp(mailbox.name, mailbox.uidvalidity,
-	   "UIDNnn", mailbox.last_uid, new_exists, 0);
+    acapmbox_setproperty(acaphandle, mailbox.name, ACAPMBOX_UIDVALIDITY,
+			 mailbox.uidvalidity);
 
     fclose(newindex);
     r = seen_reconstruct(&mailbox, (time_t)0, (time_t)0, (int (*)())0, (void *)0);
     mailbox_close(&mailbox);
+
+    if (found) {
+	/* we recurse down this directory to see if there's any mailboxes
+	   under this not in the mailboxes database */
+	dirp = opendir(".");
+
+	while ((dirent = readdir(dirp)) != NULL) {
+	    struct discovered *new;
+
+	    /* mailbox directories never have a dot in them */
+	    if (strchr(dirent->d_name, '.')) continue;
+	    if (stat(dirent->d_name, &sbuf) < 0) continue;
+	    if (!S_ISDIR(sbuf.st_mode)) continue;
+
+	    /* ok, we found a directory that doesn't have a dot in it */
+	    strcpy(fnamebuf, name);
+	    strcat(fnamebuf, ".");
+	    strcat(fnamebuf, dirent->d_name);
+
+	    /* does fnamebuf exist as a mailbox? */
+	    do {
+		r = mboxlist_lookup(fnamebuf, NULL, NULL, NULL);
+	    } while (r == IMAP_AGAIN);
+	    if (!r) continue; /* mailbox exists; it'll be reconstructed
+			         with a -r */
+	    if (r != IMAP_MAILBOX_NONEXISTENT) break; /* erg? */
+	    printf("discovered %s\n", fnamebuf);
+	    new = (struct discovered *) xmalloc(sizeof(struct discovered));
+	    new->name = strdup(fnamebuf);
+	    new->next = found->next;
+	    found->next = new;
+	}
+	closedir(dirp);
+    }
+
     return r;
 }
 
-/*
- * List of mailboxes in reconstructed mailbox list
- */
-#define NEWMBOX_GROW 1000
-char **newmbox_name = 0;
-char **newmbox_partition = 0;
-char **newmbox_acl = 0;
-int newmbox_num = 0;
-int newmbox_alloc = 0;
-
-/*
- * Insert a mailbox 'name' with 'partition' and 'acl' into
- * the being-reconstructed mailbox list
- */
-void
-newmbox_insert(name, partition, acl)
-char *name;
-char *partition;
-char *acl;
+void do_mboxlist(void)
 {
-    int low=0;
-    int high=newmbox_num-1;
-    int mid, cmp, i;
-
-    printf("%s %s\n", name, partition);
-
-    if (newmbox_num == newmbox_alloc) {
-	newmbox_alloc += NEWMBOX_GROW;
-	newmbox_name = (char **)xrealloc((char *)newmbox_name,
-					 newmbox_alloc * sizeof (char *));
-	newmbox_partition = (char **)xrealloc((char *)newmbox_partition,
-					      newmbox_alloc * sizeof (char *));
-	newmbox_acl = (char **)xrealloc((char *)newmbox_acl,
-					newmbox_alloc * sizeof (char *));
-    }
-
-    /* special-case -- appending to end */
-    if (newmbox_num == 0 || bsearch_compare(name, newmbox_name[newmbox_num-1]) > 0) {
-	newmbox_name[newmbox_num] = name;
-	newmbox_partition[newmbox_num] = partition;
-	newmbox_acl[newmbox_num] = acl;
-	newmbox_num++;
-	return;
-    }
-    
-    /* Binary-search for location */
-    while (low <= high) {
-	mid = (high - low)/2 + low;
-	cmp = bsearch_compare(name, newmbox_name[mid]);
-
-	if (cmp == 0) return;
-
-	if (cmp < 0) {
-	    high = mid - 1;
-	}
-	else {
-	    low = mid + 1;
-	}
-    }
-    
-    /* Open a slot for the new entry and insert entry into the list */
-    for (i = newmbox_num-1; i > high; i--) {
-	newmbox_name[i+1] = newmbox_name[i];
-	newmbox_partition[i+1] = newmbox_partition[i];
-	newmbox_acl[i+1] = newmbox_acl[i];
-    }
-    newmbox_num++;
-    newmbox_name[low] = name;
-    newmbox_partition[low] = partition;
-    newmbox_acl[low] = acl;
-}
-
-int
-newmbox_lookup(name)
-char *name;
-{
-    int low=0;
-    int high=newmbox_num-1;
-    int mid, cmp;
-
-    /* Binary-search for location */
-    while (low <= high) {
-	mid = (high - low)/2 + low;
-	cmp = bsearch_compare(name, newmbox_name[mid]);
-
-	if (cmp == 0) return 1;
-
-	if (cmp < 0) {
-	    high = mid - 1;
-	}
-	else {
-	    low = mid + 1;
-	}
-    }
-    return 0;
-}
-
-/* List of directories to scan for mailboxes */
-struct todo {
-    char *name;
-    char *path;
-    char *partition;
-    struct todo *next;
-} *todo_head = 0, **todo_tail = &todo_head;
-
-void
-todo_append(name, path, partition)
-char *name;
-char *path;
-char *partition;
-{
-    struct todo *newentry;
-
-    newentry = (struct todo *)xmalloc(sizeof(struct todo));
-    newentry->name = name;
-    newentry->path = path;
-    newentry->partition = partition;
-    newentry->next = 0;
-    *todo_tail = newentry;
-    todo_tail = &newentry->next;
-}
-
-void
-todo_append_hashed(char *name, char *path, char *partition)
-{
-    DIR *dirp;
-    struct dirent *dirent;
-
-    dirp = opendir(path);
-    if (!dirp) {
-	fprintf(stderr, "reconstruct: couldn't open partition %s\n", name);
-    } else while (dirent = readdir(dirp)) {
-	struct todo *newentry;
-
-	if (strchr(dirent->d_name, '.')) {
-	    continue;
-	}
-
-	newentry = (struct todo *)xmalloc(sizeof(struct todo));
-	newentry->name = xstrdup(name);
-	newentry->path = xmalloc(strlen(path) +
-				 strlen(dirent->d_name) + 2);
-	sprintf(newentry->path, "%s/%s", path, dirent->d_name);
-	newentry->partition = partition;
-	newentry->next = 0;
-	*todo_tail = newentry;
-	todo_tail = &newentry->next;
-    }
-}
-
-char *cleanacl(acl, mboxname)
-char *acl;
-char *mboxname;
-{
-    char owner[MAX_MAILBOX_NAME+1];
-    acl_canonproc_t *aclcanonproc = 0;
-    char *p;
-    char *newacl;
-    char *identifier;
-    char *rights;
-
-    /* Rebuild ACL */
-    if (!strncmp(mboxname, "user.", 5)) {
-	strcpy(owner, mboxname+5);
-	p = strchr(owner, '.');
-	if (p) *p = '\0';
-	aclcanonproc = mboxlist_ensureOwnerRights;
-    }
-    newacl = xstrdup("");
-    if (aclcanonproc) {
-	acl_set(&newacl, owner, ACL_MODE_SET, ACL_ALL,
-		(acl_canonproc_t *)0, (void *)0);
-    }
-    for (;;) {
-	identifier = acl;
-	rights = strchr(acl, '\t');
-	if (!rights) break;
-	*rights++ = '\0';
-	acl = strchr(rights, '\t');
-	if (!acl) break;
-	*acl++ = '\0';
-
-	acl_set(&newacl, identifier, ACL_MODE_SET, acl_strtomask(rights),
-		aclcanonproc, (void *)owner);
-    }
-
-    return newacl;
+    fprintf(stderr, "reconstructing mailboxes.db currently not supported\n");
+    exit(EC_USAGE);
 }
 
 /*
- * Reconstruct the mailboxes list.
+ * Cleanly shut down and exit
  */
-do_mboxlist()
+void shut_down(int code) __attribute__((noreturn));
+void shut_down(int code)
 {
-    int r;
-    const char *listfname, *newlistfname;
-    const char *startline;
-    unsigned long left;
-    const char *endline;
-    char *p;
-    char *mboxname;
-    char *partition;
-    char *acl;
-    char optionbuf[MAX_MAILBOX_NAME+1];
-    const char *root;
-    static char pathresult[MAX_MAILBOX_PATH];
-    struct mailbox mailbox;
-    char *newacl;
-    int isnewspartition;
-    DIR *dirp;
-    struct dirent *dirent;
-    struct todo *todo_next;
-    char *path;
-    FILE *newlistfile;
-    int i;
-
-    /* Lock mailbox list */
-    r = mboxlist_openlock();
-    if (r) {
-	fprintf(stderr, "reconstruct: cannot open/lock mailboxes file\n");
-	exit(1);
-    }
-
-    mboxlist_getinternalstuff(&listfname, &newlistfname,
-			      &startline, &left);
-
-    /* For each line in old mailboxes file */
-    while (endline = memchr(startline, '\n', left)) {
-	/* Copy line into malloc'ed memory; skip over line */
-	mboxname = xmalloc(endline - startline + 1);
-	strncpy(mboxname, startline, endline - startline);
-	mboxname[endline - startline] = '\0';
-	left -= endline - startline + 1;
-	startline = endline + 1;
-
-	/* Parse line */
-	partition = strchr(mboxname, '\t');
-	if (!partition) continue;
-	*partition++ = '\0';
-	acl = strchr(partition, '\t');
-	if (!acl) continue;
-	*acl++ = '\0';
-	
-	/* Check syntax of name */
-	if (mboxname_policycheck(mboxname)) continue;
-
-	/* Check partition existence */
-	if (strlen(partition) > sizeof(optionbuf)-11) {
-	    continue;
-	}
-	strcpy(optionbuf, "partition-");
-	strcat(optionbuf, partition);
-	root = config_getstring(optionbuf, (char *)0);
-	if (!root) {
-	    continue;
-	}
-	
-	/* Check mailbox exists */
-	mailbox_hash_mbox(pathresult, root, mboxname);
-	r = mailbox_open_header_path(mboxname, pathresult, "", 0,
-				     &mailbox, 1);
-	if (r) {
-	    /* Try lowercasing mailbox name */
-	    lcase(mboxname);
-	    mailbox_hash_mbox(pathresult, root, mboxname);
-
-	    r = mailbox_open_header_path(mboxname, pathresult, "", 0,
-					 &mailbox, 1);
-	}
-
-	if (r) continue;
-
-	newacl = cleanacl(acl, mboxname);
-
-	/* Store new ACL in mailbox header */
-	r = mailbox_lock_header(&mailbox);
-	if (!r) {
-	    free(mailbox.acl);
-	    mailbox.acl = xstrdup(newacl);
-	    (void) mailbox_write_header(&mailbox);
-	}
-
-	mailbox_close(&mailbox);
-	newmbox_insert(mboxname, partition, newacl);
-    }
-    
-    /* Enqueue each partition directory for scanning */
-    if (config_hashimapspool) {
-	config_scanpartition(todo_append_hashed);
-    } else {
-	config_scanpartition(todo_append);
-    }
-
-    /* Process each directory in queue */
-    while (todo_head) {
-	isnewspartition = (strcmp(todo_head->partition, "news") == 0);
-	dirp = opendir(todo_head->path);
-	if (!dirp) {
-	    free(todo_head->name);
-	    free(todo_head->path);
-	    todo_next = todo_head->next;
-	    free((char *)todo_head);
-	    todo_head = todo_next;
-	    continue;
-	}
-
-	while (dirent = readdir(dirp)) {
-	    if (!strchr(dirent->d_name, '.')) {
-		/* Ignore all-numeric files in news partitons */
-		if (isnewspartition) {
-		    p = dirent->d_name;
-		    while (*p) {
-			if (!isdigit(*p)) break;
-			p++;
-		    }
-		    if (!*p) continue;
-		}
-
-		/* Probably a directory, enqueue it */
-		mboxname = xmalloc(strlen(todo_head->name) +
-				   strlen(dirent->d_name) + 2);
-		path = xmalloc(strlen(todo_head->path) +
-			       strlen(dirent->d_name) + 2);
-		strcpy(mboxname, todo_head->name);
-		if (mboxname[0]) strcat(mboxname, ".");
-		strcat(mboxname, dirent->d_name);
-		strcpy(path, todo_head->path);
-		strcat(path, "/");
-		strcat(path, dirent->d_name);
-		todo_append(mboxname, path, todo_head->partition);
-	    }
-	    else if (!strcmp(dirent->d_name, FNAME_HEADER+1) &&
-		     !newmbox_lookup(todo_head->name) &&
-		     !mailbox_open_header_path(todo_head->name,
-					       todo_head->path, "", 0,
-					       &mailbox, 1)) {
-		r = mailbox_open_index(&mailbox);
-		if (!r) {
-		    r = mailbox_read_header_acl(&mailbox);
-		}
-		if (!r) {
-		    newmbox_insert(todo_head->name, todo_head->partition,
-				   cleanacl(mailbox.acl, todo_head->name));
-		}
-		mailbox_close(&mailbox);
-	    }
-	}
-	closedir(dirp);
-
-	todo_next = todo_head->next;
-	free(todo_head->path);
-	free((char *)todo_head);
-	todo_head = todo_next;
-    }
-	
-    newlistfile = fopen(newlistfname, "w");
-    for (i = 0; i < newmbox_num; i++) {
-	fprintf(newlistfile, "%s\t%s\t%s\n",
-		newmbox_name[i], newmbox_partition[i], newmbox_acl[i]);
-    }
-    fflush(newlistfile);
-
-    if (ferror(newlistfile) || fsync(fileno(newlistfile))) {
-	syslog(LOG_ERR, "IOERROR: writing %s: %m", newlistfname);
-	perror("writing new mailboxes list");
-	exit(1);
-    }
-    if (rename(newlistfname, listfname) == -1) {
-	syslog(LOG_ERR, "IOERROR: renaming %s: %m", listfname);
-	perror("renaming new mailboxes list");
-	exit(1);
-    }
-    
-    exit(0);
+    mboxlist_close();
+    mboxlist_done();
+    exit(code);
 }
 
 void fatal(const char* s, int code)
 {
+    static int recurse_code = 0;
+    
+    if (recurse_code) {
+	/* We were called recursively. Just give up */
+	exit(recurse_code);
+    }
+    
+    recurse_code = code;
     fprintf(stderr, "reconstruct: %s\n", s);
-    exit(code);
+    shut_down(code);
 }
-
