@@ -3,31 +3,45 @@
  * 9/21/99
  *
  *  Based upon Lutz Jaenicke's TLS patches for postfix
- */
-/***********************************************************
-        Copyright 1999 by Carnegie Mellon University
 
-                      All Rights Reserved
+ * Copyright (c) 1999-2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the name of Carnegie Mellon
-University not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
-ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
-******************************************************************/
-
-/*++
 * NAME
 *	tls
 * SUMMARY
@@ -78,6 +92,8 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
 
+#include <config.h>
+
 #ifdef HAVE_SSL
 
 /* System library. */
@@ -128,15 +144,8 @@ int     tls_clientactive = 0;	/* available or not */
 char   *tls_peer_CN = NULL;
 char   *tls_issuer_CN = NULL;
 
-char   *tls_protocol = NULL;
-const char   *tls_cipher_name = NULL;
-int	tls_cipher_usebits = 0;
-int	tls_cipher_algbits = 0;
-
-
 int var_imapd_tls_loglevel = 0;
 
-
 /* taken from OpenSSL apps/s_cb.c 
  * tim - this seems to just be giving logging messages
  */
@@ -407,8 +416,8 @@ int     tls_init_serverengine(int verifydepth,
 
     if ((!SSL_CTX_load_verify_locations(ctx, CAfile, CApath)) ||
 	(!SSL_CTX_set_default_verify_paths(ctx))) {
-	printf("TLS engine: cannot load CA data");
-	return (-1);
+	/* just a warning since this is only necessary for client auth */
+	syslog(LOG_NOTICE,"TLS engine: cannot load CA data");	
     }
     if (strlen(var_imapd_tls_cert_file) == 0)
 	s_cert_file = NULL;
@@ -420,7 +429,7 @@ int     tls_init_serverengine(int verifydepth,
 	s_key_file = var_imapd_tls_key_file;
 
     if (!set_cert_stuff(ctx, s_cert_file, s_key_file)) {
-	printf("TLS engine: cannot load cert/key data");
+	syslog(LOG_ERR,"TLS engine: cannot load cert/key data");
 	return (-1);
     }
     SSL_CTX_set_tmp_rsa_callback(ctx, tmp_rsa_cb);
@@ -482,6 +491,10 @@ int tls_start_servertls(int readfd, int writefd, int *layerbits, char **authid)
     SSL_SESSION *session;
     SSL_CIPHER *cipher;
     X509   *peer;
+    const char *tls_protocol = NULL;
+    const char *tls_cipher_name = NULL;
+    int tls_cipher_usebits = 0;
+    int tls_cipher_algbits = 0;
 
     if (!tls_serverengine)
     {		
@@ -613,18 +626,20 @@ int tls_start_servertls(int readfd, int writefd, int *layerbits, char **authid)
     }
 
 
-    if ((*authid)!=NULL)
-      syslog(LOG_NOTICE, "starttls: %s with cipher %s (%d/%d bits) authenticated as %s", tls_protocol, tls_cipher_name,
-	     tls_cipher_usebits, tls_cipher_algbits, *authid);
-    else
-      syslog(LOG_NOTICE, "starttls: %s with cipher %s (%d/%d bits) no authentication", tls_protocol, tls_cipher_name,
-	     tls_cipher_usebits, tls_cipher_algbits);
-
+    if ((*authid)!=NULL) {
+	syslog(LOG_NOTICE, "starttls: %s with cipher %s (%d/%d bits)"
+	                   " authenticated as %s", 
+	       tls_protocol, tls_cipher_name,
+	       tls_cipher_usebits, tls_cipher_algbits, 
+	       *authid);
+    } else {
+	syslog(LOG_NOTICE, "starttls: %s with cipher %s (%d/%d bits)"
+	                   " no authentication", 
+	       tls_protocol, tls_cipher_name,
+	       tls_cipher_usebits, tls_cipher_algbits);
+    }
 
     return (0);
 }
 
-
-
-
 #endif /* HAVE_SSL */
