@@ -36,10 +36,10 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- * $Id: version.h,v 1.90 2000/06/07 20:42:17 leg Exp $
+ * $Id: version.h,v 1.92 2000/06/12 02:40:21 leg Exp $
  */
 
-#define _CYRUS_VERSION "v2.0.4"
+#define _CYRUS_VERSION "v2.0.5"
 
 /* EXTRA_IDENT is a hack to add some version information for which compile
  * was used to build this version (at CMU, but we don't care what you do with
@@ -52,6 +52,6 @@
 #define CYRUS_VERSION _CYRUS_VERSION
 #endif
 
-/* CAPABILITIES are now defined here, not including sasl ones */
+/* CAPABILITIES are now defined here, not including thread and sasl ones */
 #define CAPABILITY_STRING "IMAP4 IMAP4rev1 ACL QUOTA LITERAL+ NAMESPACE " \
-	"UIDPLUS NO_ATOMIC_RENAME UNSELECT MULTIAPPEND ID"
+	"UIDPLUS NO_ATOMIC_RENAME UNSELECT MULTIAPPEND ID SORT"
