@@ -1,32 +1,50 @@
 /* actions.c -- executes the commands for timsieved
  * Tim Martin
- * $Id: actions.c,v 1.9.2.1 2000/05/12 17:54:32 leg Exp $
+ * $Id: actions.c,v 1.19 2000/05/23 20:56:50 robeson Exp $
  * 
  */
-/***********************************************************
-        Copyright 1999 by Carnegie Mellon University
-
-                      All Rights Reserved
-
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the name of Carnegie Mellon
-University not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
-THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE FOR
-ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
-OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
-******************************************************************/
-
+/*
+ * Copyright (c) 1999-2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ */
 
+#include <config.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -42,15 +60,14 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include <string.h>
 
 #include "prot.h"
-#include "config.h"
+#include "imapconf.h"
 #include "xmalloc.h"
+#include "sieve_interface.h"
 
 #include "codes.h"
 #include "actions.h"
 #include "scripttest.h"
 
-
-
 /* after a user has authentication, our current directory is their Sieve 
    directory! */
 
@@ -88,8 +105,6 @@ int actions_setuser(char *userid)
     
   snprintf(sieve_dir, 1023, "%s/%c/%s", foo, hash,userid);
 
-  printf("sievedir=%s\n",sieve_dir);
-
   result = chdir(sieve_dir);
   if (result != 0) {
       result = mkdir(sieve_dir, 0755);
@@ -103,16 +118,13 @@ int actions_setuser(char *userid)
   return TIMSIEVE_OK;
 }
 
-static int validchar(char ch)
-{
-    if (isalnum((int) ch) || (ch == '_') || (ch == '-') || (ch == ' ')) {
-	return TIMSIEVE_OK;
-    }
+/*
+ *
+ * Everything but '/' and '\0' are valid.
+ *
+ */
 
-    return TIMSIEVE_FAIL;
-}
-
-int verifyscriptname(mystring_t *name)
+int scriptname_valid(mystring_t *name)
 {
   int lup;
   char *ptr;
@@ -122,13 +134,42 @@ int verifyscriptname(mystring_t *name)
 
   ptr=string_DATAPTR(name);
 
-  for (lup=0;lup<name->len;lup++) {
-      if ( validchar(ptr[lup])!=TIMSIEVE_OK) return TIMSIEVE_FAIL;
+  for (lup=0;lup<name->len;lup++)
+  {
+      if ((ptr[lup]=='/') || (ptr[lup]=='\0'))
+	  return TIMSIEVE_FAIL;
   }
   
   return TIMSIEVE_OK;
 }
 
+int capabilities(struct protstream *conn, sasl_conn_t *saslconn)
+{
+    char *sasllist;
+    int mechcount;
+
+    /* implementation */
+    prot_printf(conn, "\"IMPLEMENTATION\" \"" SIEVED_IDENT " " SIEVED_VERSION "\"\r\n");
+    
+    /* SASL */
+    if (sasl_listmech(saslconn, NULL, 
+		    "\"SASL\" \"", " ", "\"\r\n",
+		    &sasllist,
+		    NULL, &mechcount) == SASL_OK && mechcount > 0)
+    {
+      prot_printf(conn,"%s",sasllist);
+    }
+    
+    /* Sieve capabilities */
+    prot_printf(conn,"\"SIEVE\" \"%s\"\r\n",sieve_listextensions());
+
+    /* TODO: STARTTLS */
+
+    prot_printf(conn,"OK\r\n");
+
+    return TIMSIEVE_OK;
+}
+
 int getscript(struct protstream *conn, mystring_t *name)
 {
   FILE *stream;
@@ -139,6 +180,14 @@ int getscript(struct protstream *conn, mystring_t *name)
 
   char path[1024];
 
+  result = scriptname_valid(name);
+  if (result!=TIMSIEVE_OK)
+  {
+      prot_printf(conn,"NO \"Invalid script name\"\r\n");
+      return result;
+  }
+
+
   snprintf(path, 1023, "%s.script", string_DATAPTR(name));
 
   result = stat(path, &filestats);
@@ -178,7 +227,7 @@ int getscript(struct protstream *conn, mystring_t *name)
 
   prot_printf(conn,"\r\n");
   
-  prot_printf(conn, "OK \"Success\"\r\n");
+  prot_printf(conn, "OK\r\n");
 
   return TIMSIEVE_OK;
 }
@@ -214,6 +263,7 @@ static int countscripts(char *name)
     return number;
 }
 
+
 /* save name as a sieve script */
 int putscript(struct protstream *conn, mystring_t *name, mystring_t *data)
 {
@@ -225,13 +275,20 @@ int putscript(struct protstream *conn, mystring_t *name, mystring_t *data)
   char path[1024], p2[1024];
   int maxscripts;
 
+  result = scriptname_valid(name);
+  if (result!=TIMSIEVE_OK)
+  {
+      prot_printf(conn,"NO \"Invalid script name\"\r\n");
+      return result;
+  }
+
   /* see if this would put the user over quota */
   maxscripts = config_getint("sieve_maxscripts",5);
 
   if (countscripts(string_DATAPTR(name))+1 > maxscripts)
   {
     prot_printf(conn,
-		"NO \"You are only allowed %d scripts on this server\"\r\n",
+		"NO (\"QUOTA\") \"You are only allowed %d scripts on this server\"\r\n",
 		maxscripts);
     return TIMSIEVE_FAIL;
   }
@@ -282,7 +339,7 @@ int putscript(struct protstream *conn, mystring_t *name, mystring_t *data)
   snprintf(p2, 1023, "%s.script", string_DATAPTR(name));
   rename(path, p2);
 
-  prot_printf(conn, "OK \"Success\"\r\n");
+  prot_printf(conn, "OK\r\n");
 
   return TIMSIEVE_OK;
 }
@@ -312,6 +369,7 @@ static int isactive(char *name)
 
   result=stat(filename,&filestats);
   if (result != 0) {
+      syslog(LOG_ERR, "stat failed with %m");
       return FALSE;
   }
 
@@ -328,6 +386,13 @@ int deletescript(struct protstream *conn, mystring_t *name)
   int result;
   char path[1024];
 
+  result = scriptname_valid(name);
+  if (result!=TIMSIEVE_OK)
+  {
+      prot_printf(conn,"NO \"Invalid script name\"\r\n");
+      return result;
+  }
+
   snprintf(path, 1023, "%s.script", string_DATAPTR(name));
 
   if (isactive(string_DATAPTR(name)) && (deleteactive(conn)!=TIMSIEVE_OK)) {
@@ -341,7 +406,7 @@ int deletescript(struct protstream *conn, mystring_t *name)
       return TIMSIEVE_FAIL;
   }
 
-  prot_printf(conn,"OK \"Success\"\r\n");
+  prot_printf(conn,"OK\r\n");
   return TIMSIEVE_OK;
 }
 
@@ -374,7 +439,7 @@ int listscripts(struct protstream *conn)
 	namewo[length-7]='\0';
 	
 	if (isactive(namewo)==TRUE)
-	  prot_printf(conn,"\"%s*\"\r\n", namewo);
+	  prot_printf(conn,"\"%s\" ACTIVE\r\n", namewo);
 	else
 	  prot_printf(conn,"\"%s\"\r\n", namewo);
 
@@ -383,7 +448,7 @@ int listscripts(struct protstream *conn)
     }    
   }
 
-  prot_printf(conn,"OK \"Success\"\r\n");
+  prot_printf(conn,"OK\r\n");
   
   return TIMSIEVE_OK;
 }
@@ -414,15 +479,30 @@ int setactive(struct protstream *conn, mystring_t *name)
   int result;
   char filename[1024];
 
+  result = scriptname_valid(name);
+  if (result!=TIMSIEVE_OK)
+  {
+      prot_printf(conn,"NO \"Invalid script name\"\r\n");
+      return result;
+  }
+
   if (exists(string_DATAPTR(name))==FALSE)
   {
     prot_printf(conn,"NO \"Script does not exist\"\r\n");
     return TIMSIEVE_NOEXIST;
   }
 
+  /* if script already is the active one just say ok */
+  if (isactive(string_DATAPTR(name))==TRUE) {
+      prot_printf(conn,"OK\r\n");
+      return TIMSIEVE_OK;  
+  }
+
   /* get the name of the active sieve script */
   snprintf(filename, 1023, "%s.script", string_DATAPTR(name));
 
+
+
   /* ok we want to do this atomically so let's
      - make <activesieve>.NEW as a hard link
      - rename it to <activesieve>
@@ -431,17 +511,61 @@ int setactive(struct protstream *conn, mystring_t *name)
   result = link(filename, "default.NEW");
 
   if (result!=0) {
-    prot_printf(conn, "NO \"Can't make link\"\r\n");    
-    return TIMSIEVE_FAIL;
+      syslog(LOG_ERR, "Error creating link %m");
+      prot_printf(conn, "NO \"Can't make link\"\r\n");    
+      return TIMSIEVE_FAIL;
   }
 
   result=rename("default.NEW", "default");
 
   if (result!=0) {
+      syslog(LOG_ERR, "Error renaming default.NEW to default %m");
       prot_printf(conn,"NO \"Error renaming\"\r\n");
       return TIMSIEVE_FAIL;
   }
 
-  prot_printf(conn,"OK \"Worked\"\r\n");
+
+  prot_printf(conn,"OK\r\n");
   return TIMSIEVE_OK;
 }
+
+int cmd_havespace(struct protstream *conn, mystring_t *sieve_name, unsigned long num)
+{
+    int result;
+    int maxscripts;
+    unsigned long maxscriptsize;
+
+    result = scriptname_valid(sieve_name);
+    if (result!=TIMSIEVE_OK)
+    {
+	prot_printf(conn,"NO \"Invalid script name\"\r\n");
+	return result;
+    }
+
+    /* see if the size of the script is too big */
+    maxscriptsize = config_getint("sieve_maxscriptsize", 32);
+    maxscriptsize *= 1024;
+
+    if (num > maxscriptsize)
+    {
+	prot_printf(conn,
+		    "NO (\"QUOTA\") \"Script size is too large. Max script size is %d bytes\"\r\n",
+		    maxscriptsize);
+	return TIMSIEVE_FAIL;
+    }
+
+    /* see if this would put the user over quota */
+    maxscripts = config_getint("sieve_maxscripts",5);
+
+    if (countscripts(string_DATAPTR(sieve_name))+1 > maxscripts)
+    {
+	prot_printf(conn,
+		    "NO (\"QUOTA\") \"You are only allowed %d scripts on this server\"\r\n",
+		    maxscripts);
+	return TIMSIEVE_FAIL;
+    }
+
+
+    prot_printf(conn,"OK\r\n");
+    return TIMSIEVE_OK;
+}
